/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;
import java.io.File;

/**
   This program creates an animation by repeatedly redrawing to the Java graphics window.

   This program sleeps for a short time between animation frames.
*/
public class Animation
{
   private static int fps = 20;

   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("assets/apple.obj"));
      Model m2 = new ObjSimpleModel(new File("assets/cow.obj"));
      Model m3 = new ObjSimpleModel(new File("assets/cessna.obj"));
      Model m4 = new ObjSimpleModel(new File("assets/galleon.obj"));
      Model m5 = new ObjSimpleModel(new File("assets/teapot.obj"));
      Model m6 = new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj"));
      Model m7 = new ObjSimpleModel(new File("assets/great_rhombicosidodecahedron.obj"));


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (Model m : scene.modelList)
         for (Vertex v : m.vertexList)
         {
            v.z -= 3;
         }

      // Define initial dimensions for a FrameBuffer.
      int width  = 1024;
      int height = 1024;

      // Create an AnimationFrame containing a FrameBuffer
      // with the given dimensions.
      @SuppressWarnings("serial")
      InteractiveFrame app = new AnimationFrame("Renderer 2", width, height, fps)
      {
         // Implement the ActionListener interface.
         @Override public void actionPerformed(ActionEvent e)
         {
            // update the scene
            for (Vertex v : scene.modelList.get(0).vertexList)
            {
               v.x += 0.3;  // move right
               v.y += 0.1;  // move up
               v.z -= 0.1;  // move back
            }
            for (Vertex v : scene.modelList.get(1).vertexList)
            {
               v.x -= 0.3;  // move left
               v.y -= 0.1;  // move down
               v.z -= 0.1;  // move back
            }
            for (Vertex v : scene.modelList.get(2).vertexList)
            {
               v.x += 0.01; // move right
            }
            // Render again.
            FrameBuffer fb = this.fbp.getFrameBuffer();
            fb.clearFB(Color.darkGray);
            Pipeline.render(scene, fb);
            fbp.update();
            repaint();
         }

         // Implement part of the KeyListener interface.
         // This can be used to change the frame rate.
         @Override public void keyTyped(KeyEvent e)
         {
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );

            // Get the new size of the FrameBufferPanel.
            int w = this.fbp.getWidth();
            int h = this.fbp.getHeight();

            // Create a new FrameBuffer that fits the new window size.
            FrameBuffer fb = new FrameBuffer(w, h);
            this.fbp.setFrameBuffer(fb);
            fb.clearFB(Color.darkGray);
            Pipeline.render(scene, fb);
            fbp.update();
            repaint();
         }
      };
   }
}
